import { __awaiter } from 'tslib';
import * as i1 from '@angular/common/http';
import { HttpErrorResponse, HttpParams, HttpClientModule } from '@angular/common/http';
import * as i0 from '@angular/core';
import { InjectionToken, Injectable, Optional, Inject, PLATFORM_ID, ViewContainerRef, Component, ViewChild, NgModule } from '@angular/core';
import * as i3 from '@angular/platform-browser';
import { makeStateKey } from '@angular/platform-browser';
import { isPlatformServer, DOCUMENT } from '@angular/common';
import { BehaviorSubject, of, from, EMPTY } from 'rxjs';
import { exhaustMap, tap, catchError, map } from 'rxjs/operators';
import { RESPONSE } from '@nguniversal/express-engine/tokens';
import * as i2 from '@angular/router';

const WAGTAIL_CONFIG = new InjectionToken('Wagtail Config');

class StatusCodeService {
    constructor(_response) {
        this._response = _response;
    }
    setStatus(code, message) {
        if (this._response) {
            this._response.statusCode = code;
            this._response.statusMessage = message;
        }
    }
    redirect(url, isPermanant = true) {
        if (isPermanant) {
            this.setStatus(301, 'Moved Permanently');
        }
        else {
            this.setStatus(302, 'Found');
        }
        this._response.setHeader('Location', url);
    }
}
StatusCodeService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: StatusCodeService, deps: [{ token: RESPONSE, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
StatusCodeService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: StatusCodeService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: StatusCodeService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [RESPONSE]
                }] }]; } });

const CMS_DOMAIN_KEY = makeStateKey('Wagtail Domain');
const CMS_SITE_ID_KEY = makeStateKey('Wagtail Site ID');
class WagtailService {
    constructor(http, statusCodeService, state, title, meta, doc, config, platformId) {
        this.http = http;
        this.statusCodeService = statusCodeService;
        this.state = state;
        this.title = title;
        this.meta = meta;
        this.doc = doc;
        this.config = config;
        this.platformId = platformId;
        this.page = new BehaviorSubject(null);
        this.currentPageComponent = this.page.pipe(exhaustMap((page) => {
            return this.getComponentForPage(page);
        }));
    }
    /** Strip out query params and #anchor */
    sanitizePath(path) {
        return path.split('?')[0].split('#')[0];
    }
    getPageForUrl(url) {
        var _a;
        const pathname = this.sanitizePathname(url);
        const params = new URLSearchParams({ html_path: pathname });
        const siteId = this.getWagtailSiteID();
        if (siteId)
            params.set('site', siteId.toString());
        return this.http
            .get(`${this.getWagtailDomain()}${(_a = this.config.pagesApiPath) !== null && _a !== void 0 ? _a : '/api/v2/pages/detail_by_path/'}?${params.toString()}`)
            .pipe(tap((page) => this.setMeta(page)), catchError((err) => {
            if (err instanceof HttpErrorResponse && err.status === 404) {
                return of(null);
            }
            throw err;
        }));
    }
    /**
     * Get the Wagtail Domain
     * Set the env var CMS_DOMAIN in node to override the domain
     * Otherwise it will default to "" meaning the same domain angular runs in.
     */
    getWagtailDomain() {
        let cmsDomain = this.state.get(CMS_DOMAIN_KEY, '');
        if (cmsDomain) {
            return cmsDomain;
        }
        if (this.config.apiDomain) {
            cmsDomain = this.config.apiDomain;
        }
        else if (isPlatformServer(this.platformId)) {
            const envCmsDomain = process.env.CMS_DOMAIN;
            if (envCmsDomain) {
                cmsDomain = envCmsDomain;
            }
        }
        this.state.set(CMS_DOMAIN_KEY, cmsDomain);
        return cmsDomain;
    }
    /** Get Wagtail Site ID which may be configured by a Node env var or from the module config  */
    getWagtailSiteID() {
        let cmsId = this.state.get(CMS_SITE_ID_KEY, null);
        if (typeof cmsId === 'number') {
            return cmsId;
        }
        if (this.config.siteId) {
            cmsId = this.config.siteId;
        }
        else if (isPlatformServer(this.platformId)) {
            const envcmsId = process.env.WAGTAIL_SITE_ID;
            if (envcmsId) {
                cmsId = parseInt(envcmsId);
            }
        }
        this.state.set(CMS_SITE_ID_KEY, cmsId);
        return cmsId;
    }
    getRedirectForUrl(url) {
        const siteId = this.getWagtailSiteID();
        const params = Object.assign({ old_path: url }, (siteId && {
            site: siteId.toString(),
        }));
        return this.http.get(`${this.getWagtailDomain()}/api/redirects/`, { params });
    }
    getComponentForPage(page) {
        return __awaiter(this, void 0, void 0, function* () {
            if (page === null) {
                return;
            }
            else if (page === '404') {
                this.statusCodeService.setStatus(404, 'Not Found');
                return this.config.notFoundComponent;
            }
            const route = this.config.pageTypes.find((route) => route.type === page.meta.type);
            if (!route) {
                return this.config.notFoundComponent;
            }
            if ('component' in route) {
                return route.component;
            }
            else {
                return yield route.loadComponent();
            }
        });
    }
    /** Support for wagtail-headless-preview API */
    getPreviewData(contentType, token) {
        const url = this.getWagtailDomain() + '/api/v2/page_preview/1/';
        const params = new HttpParams({
            fromObject: {
                content_type: contentType,
                token: token,
                format: 'json',
            },
        });
        return this.http.get(url, { params });
    }
    sanitizePathname(path) {
        return path.split(/[?#]/)[0];
    }
    /** Set title and description meta tag for SEO */
    setMeta(data) {
        if (data.meta.seo_title) {
            this.title.setTitle(data.meta.seo_title);
        }
        else {
            this.title.setTitle(data.title);
        }
        if (data.meta.search_description) {
            this.meta.addTag({
                name: 'description',
                content: data.meta.search_description,
            });
        }
        if (this.config.setCanonicalURL) {
            this.createLinkForCanonicalURL(data.meta.html_url);
        }
    }
    createLinkForCanonicalURL(url) {
        const link = this.doc.createElement('link');
        link.setAttribute('rel', 'canonical');
        this.doc.head.appendChild(link);
        link.setAttribute('href', url);
    }
}
WagtailService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: WagtailService, deps: [{ token: i1.HttpClient }, { token: StatusCodeService }, { token: i3.TransferState }, { token: i3.Title }, { token: i3.Meta }, { token: DOCUMENT }, { token: WAGTAIL_CONFIG }, { token: PLATFORM_ID }], target: i0.ɵɵFactoryTarget.Injectable });
WagtailService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: WagtailService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: WagtailService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }], ctorParameters: function () { return [{ type: i1.HttpClient }, { type: StatusCodeService }, { type: i3.TransferState }, { type: i3.Title }, { type: i3.Meta }, { type: Document, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [WAGTAIL_CONFIG]
                }] }, { type: Object, decorators: [{
                    type: Inject,
                    args: [PLATFORM_ID]
                }] }]; } });

class WagtailComponent {
    constructor(wagtail, cfr, injector) {
        this.wagtail = wagtail;
        this.cfr = cfr;
        this.injector = injector;
    }
    ngAfterViewInit() {
        this.wagtail.currentPageComponent.subscribe((component) => {
            if (component) {
                const factory = this.cfr.resolveComponentFactory(component);
                this.outlet.clear();
                this.outlet.createComponent(factory, undefined, this.injector);
            }
        });
    }
}
WagtailComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: WagtailComponent, deps: [{ token: WagtailService }, { token: i0.ComponentFactoryResolver }, { token: i0.Injector }], target: i0.ɵɵFactoryTarget.Component });
WagtailComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.2.1", type: WagtailComponent, selector: "lib-angular-wagtail", viewQueries: [{ propertyName: "outlet", first: true, predicate: ["wagtailOutlet"], descendants: true, read: ViewContainerRef }], ngImport: i0, template: `<ng-container #wagtailOutlet></ng-container>`, isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: WagtailComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'lib-angular-wagtail',
                    template: `<ng-container #wagtailOutlet></ng-container>`,
                }]
        }], ctorParameters: function () { return [{ type: WagtailService }, { type: i0.ComponentFactoryResolver }, { type: i0.Injector }]; }, propDecorators: { 
        // @ts-ignore
        outlet: [{
                type: ViewChild,
                args: ['wagtailOutlet', { read: ViewContainerRef }]
            }] } });

class GetPageDataResolverService {
    constructor(wagtail, router, platformId, statusCodeService) {
        this.wagtail = wagtail;
        this.router = router;
        this.platformId = platformId;
        this.statusCodeService = statusCodeService;
    }
    resolve(route, state) {
        if (route.data.preview) {
            const contentType = route.queryParams.content_type;
            const token = route.queryParams.token;
            if (contentType && token) {
                return this.wagtail.getPreviewData(contentType, token).pipe(exhaustMap((page) => {
                    if (page) {
                        return from(this.wagtail.getComponentForPage(page)).pipe(map(() => {
                            this.wagtail.page.next(page);
                            return page;
                        }));
                    }
                    return EMPTY;
                }));
            }
        }
        return this.wagtail.getPageForUrl(state.url).pipe(exhaustMap((page) => {
            if (page === null) {
                return this.wagtail.getRedirectForUrl(state.url).pipe(exhaustMap((redirects) => {
                    if (redirects.length > 0) {
                        this.redirect(redirects[0]);
                        return of(null);
                    }
                    this.wagtail.page.next('404');
                    return of('404');
                }));
            }
            // We need to force any lazy loaded code to load in the resolver, but it should
            // return only the page data.
            return from(this.wagtail.getComponentForPage(page)).pipe(map(() => {
                this.wagtail.page.next(page);
                return page;
            }));
        }));
    }
    /**
     * On server, send redirect status code
     * On browser, change window.location
     */
    redirect(redirect) {
        if (isPlatformServer(this.platformId)) {
            this.statusCodeService.redirect(redirect.link, redirect.is_permanent);
        }
        else {
            const path = redirect.link;
            if (path.startsWith('http')) {
                if (window.location.href === path) {
                    console.warn('Refused to redirect to self');
                }
                else {
                    window.location.href = path;
                }
            }
            else {
                this.router.navigate([path]);
            }
        }
    }
}
GetPageDataResolverService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: GetPageDataResolverService, deps: [{ token: WagtailService }, { token: i2.Router }, { token: PLATFORM_ID }, { token: StatusCodeService }], target: i0.ɵɵFactoryTarget.Injectable });
GetPageDataResolverService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: GetPageDataResolverService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: GetPageDataResolverService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }], ctorParameters: function () { return [{ type: WagtailService }, { type: i2.Router }, { type: Object, decorators: [{
                    type: Inject,
                    args: [PLATFORM_ID]
                }] }, { type: StatusCodeService }]; } });

class WagtailModule {
    static forRoot(config) {
        return {
            ngModule: WagtailModule,
            providers: [
                GetPageDataResolverService,
                { provide: WAGTAIL_CONFIG, useValue: config },
            ],
        };
    }
}
WagtailModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: WagtailModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
WagtailModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: WagtailModule, declarations: [WagtailComponent], imports: [HttpClientModule], exports: [WagtailComponent] });
WagtailModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: WagtailModule, providers: [StatusCodeService], imports: [[HttpClientModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.1", ngImport: i0, type: WagtailModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [WagtailComponent],
                    imports: [HttpClientModule],
                    providers: [StatusCodeService],
                    exports: [WagtailComponent],
                }]
        }] });

/*
 * Public API Surface of angular-wagtail
 */

/**
 * Generated bundle index. Do not edit.
 */

export { GetPageDataResolverService, StatusCodeService, WagtailComponent, WagtailModule, WagtailService };
//# sourceMappingURL=angular-wagtail.js.map
